;*****************************************************************************
;*
;* MODULE	: cstart.asm
;*
;* DESCRIPTION	: C startup code for non-extended C166 architecture CPU's
;*		  (e.g. SAB 80C166, GOLD)
;*
;*		- Processor initialization.
;*		- Initialization of static variables in internal or external
;*		  ram is done in '_c_init.asm'.
;*		- Call the user program: main().
;*		- On exit cpu is set in idle mode.
;*
;* COPYRIGHTS	: 1998 TASKING, Inc.
;*
;*****************************************************************************

$CHECKCPU16
$include(head.asm)
$include(_c_init.asm)
	
NAME	CSTART			; module name.
	
@IF( @EQS(@MODEL,"LARGE") | @EQS(@MODEL,"SMALL") )
  @IF( @DEFINED( @CALLEINIT))
	EXTERN	@CALLEINIT:FAR		; optional call before EINIT
  @ENDI
  @IF( @DEFINED( @CALLINIT))
	EXTERN	@CALLINIT:FAR		; optional call before _main
  @ENDI
	EXTERN	_main:FAR		; start label user program.
@ELSE
  @IF( @DEFINED( @CALLEINIT))
	EXTERN	@CALLEINIT:NEAR		; optional call before EINIT
  @ENDI
  @IF( @DEFINED( @CALLINIT))
	EXTERN	@CALLINIT:NEAR		; optional call before _main
  @ENDI
	EXTERN	_main:NEAR		; start label user program.
@ENDI

	PUBLIC	__IDLE			; cstart end
	PUBLIC	__EXIT			; address to jump to on 'exit()'.

@IF( @EX_AB )
  @IF( @EQS(@MODEL,"LARGE") | @EQS(@MODEL,"SMALL") )
	EXTERN	_exit:FAR		; exit()
  @ELSE
	EXTERN	_exit:NEAR		; exit()
  @ENDI
@ENDI

	; Value definitions for System Configuration Register : SYSCON

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. SYSCON[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC ) )
	@SET(__MCTC, 1)		; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).
				; At least 1 waitstate is needed
				; for EVA166 to meet the EPROM specification.

	; The Read/Write Signal Delay is 0.5 or 0 State Times. SYSCON.4
	; If multiplexed bus mode is selected RWDC must be set to 0 for EVA166.
@IF( ! @DEFINED( __RWDC ) )
  @IF( @EVA && @MUXBUS )
	@SET(__RWDC, 0)		; 0 = Delay Time (Reset value)
  @ELSE
	@SET(__RWDC, 1)		; 1 = No Delay Time
  @ENDI
@ENDI

	; Memory Tri-state is extended by either 1 or 0 State Times. SYSCON.5
@IF( ! @DEFINED( __MTTC ) )
	@SET(__MTTC, 0)		; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time
				; It is expected that the Tri-state
				; delay is needed for the memory.

	; BTYP is read only for external bus configurations. SYSCON[7..6]

	; System Clock Output (CLKOUT) Enable bit. SYSCON.8
@IF( ! @DEFINED( __CLKEN ) )
	@SET(__CLKEN, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Byte High Enable (BHE#) pin control bit. SYSCON.9
@IF( ! @DEFINED( __BYTDIS ) )
	@SET(__BYTDIS, 0)	; 0 = Enabled (Reset value)
@ENDI				; 1 = Disabled

	; Internal ROM Access Enable (Read Only). SYSCON.10

	; Segmentation Disable control bit. SYSCON.11
@IF( ! @DEFINED( __SGTDIS ) )
  @IF( @EQS( @MODEL, "TINY" )  &  NOT (@EVA) )
	@SET(__SGTDIS, 1)	; 1 = Disable segmented memory mode
				;     for TINY model and NOT EVA
  @ELSE
	@SET(__SGTDIS, 0)	; 0 = Reset value is segmentation enabled
  @ENDI
@ENDI

	; READY# Input Enable control bit. SYSCON.12
@IF( ! @DEFINED( __RDYEN ) )
	@SET(__RDYEN, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Stack Size selection of between 32 and 256 words. SYSCON[14..13]
@IF( ! @DEFINED( __STKSZ ) )	; System stack size
	@SET(__STKSZ, 0)	; 0 = 256 words (Reset value)
@ENDI				; 1 = 128 words
				; 2 =  64 words
				; 3 =  32 words

	; Process SYSCON low byte and high byte values.
	SYSC_L  	EQU	((@__MTTC << 5) | (@__RWDC << 4) | ((~@__MCTC) & 000FH))
	SYSC_H  	EQU  	((@__STKSZ << 5) | (@__RDYEN <<4) | (@__SGTDIS << 3) | (@__BYTDIS << 1) | @__CLKEN)
	SYSC_M_L	EQU	03FH	; Mask low byte SYSCON.
	SYSC_M_H	EQU	07BH	; Mask high byte SYSCON.

@IF( @EQS( @MODEL, "SMALL") | @EQS( @MODEL, "LARGE") )
EXTERN __C_INIT:FAR
@ELSE
EXTERN __C_INIT:NEAR
@ENDI		

;*****************************************************************************
;* __CSTART
;*****************************************************************************
__CSTART_PR	SECTION CODE WORD PUBLIC 'CPROGRAM'
__CSTART 	PROC TASK __CSTART_TASK INTNO __CSTART_INUM = 00H

	DISWDT				; Disable watchdog timer.

						; Set SYSCON register.
	BFLDL	SYSCON, #SYSC_M_L, #(SYSC_L AND SYSC_M_L)
	BFLDH	SYSCON, #SYSC_M_H, #(SYSC_H AND SYSC_M_H)
	MOV	STKOV,	#?SYSSTACK_BOTTOM + 6*2	; Set stack underflow pointer.
	MOV	STKUN,	#?SYSSTACK_TOP		; Set stack overflow pointer.
	MOV	SP,	#?SYSSTACK_TOP		; Set stack pointer.

	MOV	CP, 	#RBANK		; Set context pointer.
	NOP			
	MOV	R0, #BOTTOM_BITRAM	; Set R0
	BSET	P3.13			; Set WR output high.
	BSET	DP3.13			; enable output of WR strobe.
@IF( @DEFINED( @CALLEINIT))
@_CALL( @CALLEINIT, R1)		
@ENDI
	EINIT				; End of initialization

; start of init function
@_CALL(__C_INIT, R1)		
; end of init function

  @IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
	MOV	R0, #POF ?USRSTACK_TOP	; set user stack pointer
	BSET	R0.0EH			; User stack uses DPP1
  @ELSE
	MOV	R0, #?USRSTACK_TOP	; set user stack pointer
  @ENDI

  @IF( @EVA )
	BSET	IEN			; allow monitor to break application
  @ENDI

	MOV	R12, #0			; set argc to 0
	MOV	R13, #0			;
	MOV	R14, #0			; set argv[] to 0


@IF( @DEFINED( @CALLINIT))
@_CALL( @CALLINIT, R1)		
@ENDI
@_CALL( _main, R1)
								
; The exit() function causes normal program termination to occur. First, all 
; functions registered by the atexit() function are called in the reverse 
; order. Next, all open streams with unwritten buffered data are flushed, all 
; open streams are closed and all files created by the tmpfile() function are 
; removed. The status value passed to exit is returned in R4.
__EXIT: LABEL FAR			; the exit() or abort() function jumps
					; to this entry.
@IF( @EX_AB )
					; restore (host) environment.
	MOV	SP, #?SYSSTACK_TOP	; restore stack pointer.
  @IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
	MOV	R0, #POF ?USRSTACK_TOP	; restore user stack pointer
	BSET	R0.0EH			; User stack uses DPP1
  @ELSE
	MOV	R0, #?USRSTACK_TOP	; restore user stack pointer
  @ENDI
  @IF( @FLOAT )
	MOV	R1, #?FPSTKUN		; restore floating point stack pointer.
	MOV	?FPSP, R1		; ?FPSP normally initialized via 
					; C166_INIT.
  @ENDI	
@ENDI

__IDLE: IDLE				; Power down CPU until peripheral inter-
					; rupt or external interrupt occurs.
	JMPR	CC_UC, __IDLE		; set idle mode again.
	RETV				; Virtual return.
__CSTART	ENDP
__CSTART_PR	ENDS

@IF( @EQS(@MODEL,"TINY") | @EQS(@MODEL,"SMALL") )
C166_US	SECTION	LDAT WORD GLBUSRSTACK 'CUSTACK'
@ENDI
@IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
C166_US	SECTION	DATA WORD GLBUSRSTACK 'CUSTACK'
@ENDI
@IF( @_USRSTACK & ( @EQS(@MODEL,"SMALL") | @EQS(@MODEL,"LARGE") ) )
  @IF( @EQS( @MODEL, "SMALL" ) | @EQS( @MODEL, "LARGE" ) )
	DS	2+4		; Allocate a user stack of at least 2 bytes +
				; return label main (__MAIN_RET).
  @ELSE
	DS	2+2		; Allocate a user stack of at least 2 bytes +
				; return label main (__MAIN_RET).
  @ENDI
@ELSE
	DS	2		; Allocate a user stack of at least 2 bytes
@ENDI
C166_US	ENDS

@IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
C166_DGROUP	DGROUP	__DUMMY
__DUMMY	SECTION DATA WORD PUBLIC 'CNEAR'
__DUMMY	ENDS
@ENDI

RBANK	REGDEF R0-R15			; Register usage
	SSKDEF	@__STKSZ		; System stack size
	
	END
