;*****************************************************************************
;*
;* MODULE	: cstartx.asm
;*
;* DESCRIPTION	: C startup code for extended C166 architecture CPU's
;*		  (e.g. C161x, C163, C164xx, C165, C167xx, ST10-262).
;*
;*		- Processor initialization.
;*		- Initialization of static variables in internal or external
;*		  ram is done in '_c_init.asm'.
;*		- Call the user program: main().
;*		- On exit cpu is set in idle mode.
;*
;* COPYRIGHTS	: 1998 TASKING, Inc.
;*
;*****************************************************************************

$CHECKCPU16
$NOMOD166			; disable the internal set of SAB 80C166 SFRs
$STDNAMES(reg167.def)		; use extended set of SFR's (BUSCON0 etc.)

$include(head.asm)
$include(_c_init.asm)		; initialization of static variables in internal or external ram.
	
	NAME	CSTART		; module name.
	
@IF( @EQS(@MODEL,"LARGE") | @EQS(@MODEL,"SMALL") )
  @IF( @DEFINED( @CALLEINIT))
	EXTERN	@CALLEINIT:FAR		; optional call before EINIT
  @ENDI
  @IF( @DEFINED( @CALLINIT))
	EXTERN	@CALLINIT:FAR		; optional call before _main
  @ENDI
	EXTERN	_main:FAR		; start label user program.
@ELSE
  @IF( @DEFINED( @CALLEINIT))
	EXTERN	@CALLEINIT:NEAR		; optional call before EINIT
  @ENDI
  @IF( @DEFINED( @CALLINIT))
	EXTERN	@CALLINIT:NEAR		; optional call before _main
  @ENDI
	EXTERN	_main:NEAR		; start label user program.
@ENDI

	PUBLIC	__IDLE			; cstart end
	PUBLIC	__EXIT			; address to jump to on 'exit()'.

@IF( @EX_AB )
  @IF( @EQS(@MODEL,"LARGE") | @EQS(@MODEL,"SMALL") )
	EXTERN	_exit:FAR		; exit()
  @ELSE
	EXTERN	_exit:NEAR		; exit()
  @ENDI
@ENDI

@IF( @EQS( @MODEL, "SMALL") | @EQS( @MODEL, "LARGE") )
EXTERN __C_INIT:FAR
@ELSE
EXTERN __C_INIT:NEAR
@ENDI		

	; Value definitions for System Configuration Register : SYSCON

        ; XBUS Peripheral Share Mode Control (XPER-SHARE) bit. SYSCON.0
@IF( ! @DEFINED( __XPER_SHARE ) )
	@SET(__XPER_SHARE, 0)	; 0 = Disable external accesses to XBUS peripherals
@ENDI				; 1 = XBUS peripherals accessible via 
				;     external bus during hold mode

	; Visible Mode Control (VISIBLE) bit. SYSCON.1
@IF( ! @DEFINED( __VISIBLE ) )
	@SET(__VISIBLE, 0)	; 0 = Accesses to XBUS peripherals internally
@ENDI				; 1 = XBUS peripherals accessible visible on
                                ;     external pins

	; XBUS Peripheral (XPEN) Enable bit. SYSCON.2
@IF( ! @DEFINED( __XPEN ) )
	@SET(__XPEN, 0)		; 0 = On-chip X-Peripherals accesses
@ENDI				;     are done internally
				; 1 = On-chip X-Peripherals Enable

	; Bidirectional Reset (BDRSTEN) Enable bit. SYSCON.3
@IF( ! @DEFINED( __BDRSTEN ) )
	@SET(__BDRSTEN, 0)	; 0 = Pin RSTIN is input only
@ENDI				; 1 = Pin RSTIN pulled low during internal
				;     reset after software or WDT reset

	; Oscillator Watchdog (OWDDIS) Disable bit. SYSCON.4
@IF( ! @DEFINED( __OWDDIS ) )
	@SET(__OWDDIS, 1)	; 0 = Enable on-chip oscillator watchdog
@ENDI				; 1 = Disable on-chip oscillator watchdog

	; Chip Selected Configuration Control (CSCFG) bit. SYSCON.6
@IF( ! @DEFINED( __CSCFG ) )
	@SET(__CSCFG, 0)	; 0 = Latched CS mode
@ENDI				; 1 = Unlatched CS mode

	; Write Configuration Mode Control Bit (CLKOUT) Enable bit. SYSCON.7
@IF( ! @DEFINED( __WRCFG ) )
	@SET(__WRCFG, 0)	; 0 = Normal operation of WR# abd BHE# (Reset)
@ENDI				; 1 = WR# acts as WRL#, BHE# acts as WRH#

	; System Clock Output (CLKOUT) Enable bit. SYSCON.8
@IF( ! @DEFINED( __CLKEN ) )
	@SET(__CLKEN, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Byte High Enable (BHE#) pin control bit. SYSCON.9
@IF( ! @DEFINED( __BYTDIS ) )
	@SET(__BYTDIS, 0)	; 0 = Enabled (Reset value)
@ENDI				; 1 = Disabled

	; Internal ROM Access (ROMEN) Enable bit. SYSCON.10
				; Reset value determined by EA pin
@IF( ! @DEFINED( __ROMEN ) )
	@SET(__ROMEN, 0)	; 0 = Disable Internal ROM
@ENDI				; 1 = Enable Internal ROM

	; Segmentation Disable control bit. SYSCON.11
@IF( ! @DEFINED( __SGTDIS ) )
  @IF( @EQS( @MODEL, "TINY" )  &  NOT (@EVA) )
	@SET(__SGTDIS, 1)	; 1 = Disable segmented memory mode
				;     for TINY model and NOT EVA
  @ELSE
	@SET(__SGTDIS, 0)	; 0 = Reset value is segmentation enabled
  @ENDI
@ENDI

	; ROM Segment Mapping control bit. SYSCON.12
@IF( ! @DEFINED( __ROMS1 ) )
	@SET(__ROMS1, 0)	; 0 = Map internal ROM to segment 0 (Reset)
@ENDI				; 1 = Map internal ROM to segment 1

	; Stack Size selection of between 32 and 512 words. SYSCON[15..13]
@IF( ! @DEFINED( __STKSZ ) )	; System stack size
	@SET(__STKSZ, 0)	; 0 = 256 words (Reset value)
@ENDI				; 1 = 128 words
				; 2 =  64 words
				; 3 =  32 words
				; 4 = 512 words
				; 7 = No wrapping

	; Process SYSCON low byte and high byte values.
	SYSC_L  	EQU	((@__WRCFG<<7) | (@__CSCFG<<6) | (@__OWDDIS<<4) | (@__BDRSTEN<<3) | (@__XPEN<<2) | (@__VISIBLE<<1) | @__XPER_SHARE)
	SYSC_H  	EQU  	((@__STKSZ<<5) | (@__ROMS1<<4) | (@__SGTDIS<<3) | (@__ROMEN<<2) | (@__BYTDIS<<1) | @__CLKEN)
	SYSC_M_L	EQU	0DFH	; Mask low byte SYSCON.
	SYSC_M_H	EQU	0FFH	; Mask high byte SYSCON.


	; Value definitions for System Configuration Register : SYSCON1

	; Sleep Mode Configuration (SLEEPCON) field. SYSCON1[0..1]
@IF( ! @DEFINED( __SLEEPCON ) )
	@SET(__SLEEPCON, 0 )	; 0 = Normal IDLE mode
@ENDI				; 1 = SLEEP mode, with RTC running
				; 3 = SLEEP mode, with RTC and oscillator stopped

	; Process SYSCON1 values.
	SYSC1	  	EQU	(@__SLEEPCON)


	; Value definitions for System Configuration Register : SYSCON2

	; SYSCON Release Function (SYSRLS) field. SYSCON2[0..3]
				; Unlock sequence field after EINIT

	; Power Down Control (PDCON) field. SYSCON2[4..5]
@IF( ! @DEFINED( __PDCON ) )
	@SET(__PDCON, 0 )	; 0 = RTC on, Ports on (Reset)
@ENDI				; 1 = RTC on, Ports off
				; 2 = RTC off, Ports on
				; 3 = RTC off, Ports off

	; RTC Clock Source (RCS) bit. SYSCON2.6
@IF( ! @DEFINED( __RCS ) )
	@SET(__RCS, 0 )		; 0 = Main oscillator
@ENDI				; 1 = Reserved

	; SDD Clock Source (SCS) bit. SYSCON2.7
@IF( ! @DEFINED( __SCS ) )
	@SET(__SCS, 0 )		; 0 = Main oscillator
@ENDI				; 1 = Reserved

	; Clock State Control (CLKCON) field. SYSCON2[8..9]
@IF( ! @DEFINED( __CLKCON ) )
	@SET(__CLKCON, 0 )	; 0 = Run on configured basic frequency
@ENDI				; 1 = Run on slow down frequency, PLL ON
				; 2 = Run on slow down frequency, PLL OFF

	; Reload Counter Value for Slowdown Divider (CLKREL) field. SYSCON2[10..14]
@IF( ! @DEFINED( __CLKREL ) )
	@SET(__CLKREL, 0 )	; Reload value
@ENDI

	; Clock Signal Status (CLKLOCK) bit is read only. SYSCON2.15

	; Process SYSCON2 low byte and high byte values.
	SYSC2_L  	EQU	((@__SCS<<7) | (@__RCS<<6) | ((@__PDCON&0003h)<<4))
	SYSC2_H  	EQU  	(((@__CLKREL&001fh)<<2) | (@__CLKCON&0003h))
	SYSC2_M_L	EQU	0F0H	; Mask low byte SYSCON2.
	SYSC2_M_H	EQU	07FH	; Mask high byte SYSCON2.


	; Value definitions for System Configuration Register : SYSCON3

	; Analog/Digital Converter (ADCDIS) enable bit. SYSCON3.0
@IF( ! @DEFINED( __ADCDIS ) )
	@SET(__ADCDIS, 0)	; 0 = Disable ADC convertor
@ENDI				; 1 = Enable ADC convertor

	; USART ASC0 (ASC0DIS) enable bit. SYSCON3.1
@IF( ! @DEFINED( __ASC0DIS ) )
	@SET(__ASC0DIS, 0)	; 0 = Disable USART ASC0
@ENDI				; 1 = Enable USART ASC0

	; Synchronous Serial Channel SSC (SSCDIS) enable bit. SYSCON3.2
@IF( ! @DEFINED( __SSCDIS ) )
	@SET(__SSCDIS, 0 )	; 0 = Disable Synchronous Serial Channel SSC
@ENDI				; 1 = Enable Synchronous Serial Channel SSC

	; General Purpose Timer Blocks (GPTDIS) enable bit. SYSCON3.3
@IF( ! @DEFINED( __GPTDIS ) )
  @IF( ! @DEFINED( __GPT1DIS ) )
	@SET(__GPTDIS, 0 )	; 0 = Disable General Purpose Timer Blocks
				; 1 = Enable General Purpose Timer Blocks
  @ELSE
	@SET(__GPTDIS, @__GPT1DIS)
  @ENDI
@ENDI

	; General Purpose Timer Block 2 (GPT2DIS) enable bit. SYSCON3.4
@IF( @DEFINED( __GPT2DIS ) )
	@SET(__SYSCON3_4, @__GPT2DIS )	; 0 = Disable General Purpose Timer Block 2
@ELSE					; 1 = Enable General Purpose Timer Block 2
	@SET(__SYSCON3_4, 0 )
@ENDI

	; On-chip Data Flash Memory Module (DFMDIS) enable bit. SYSCON3.4
@IF( @DEFINED( __DFMDIS ) )
	@SET(__SYSCON3_4, @__DFMDIS )	; 0 = Disable on-chip Data Flash Memory Module
@ENDI					; 1 = Enable on-chip Data Flash Memory Module

	; On-chip Flash Memory Module (FMDIS) enable bit. SYSCON3.5
@IF( @DEFINED( __FMDIS ) )
	@SET(__SYSCON3_5, @__FMDIS )	; 0 = Disable on-chip Flash Memory Module
@ELSE					; 1 = Enable on-chip Flash Memory Module
	@SET(__SYSCON3_5, 0 )
@ENDI

	; On-chip Program Flash Memory Module (PFMDIS) enable bit. SYSCON3.5
@IF( @DEFINED( __PFMDIS ) )
	@SET(__SYSCON3_5, @__PFMDIS )	; 0 = Disable on-chip Program Flash Memory Module
@ENDI					; 1 = Enable on-chip Program Flash Memory Module

	; CAPCOM Unit 1 (CC1DIS) enable bit. SYSCON3.6
@IF( ! @DEFINED( __CC1DIS ) )
	@SET(__CC1DIS, 0 )	; 0 = Disable CAPCOM Unit 1
@ENDI				; 1 = Enable CAPCOM Unit 1

	; CAPCOM Unit 2 (CC2DIS) enable bit. SYSCON3.7
@IF( ! @DEFINED( __CC2DIS ) )
	@SET(__CC2DIS, 0 )	; 0 = Disable CAPCOM Unit 2
@ENDI				; 1 = Enable CAPCOM Unit 2

	; CAPCOM Unit 6 (CC6DIS) enable bit. SYSCON3.8
@IF( ! @DEFINED( __CC6DIS ) )
	@SET(__CC6DIS, 0 )	; 0 = Disable CAPCOM Unit 6
@ENDI				; 1 = Enable CAPCOM Unit 6

	; PWM Unit (PWMDIS) enable bit. SYSCON3.9
@IF( ! @DEFINED( __PWMDIS ) )
	@SET(__PWMDIS, 0 )	; 0 = Disable PWM Unit
@ENDI				; 1 = Enable PWM Unit

	; USART ASC1 (ASC1DIS) enable bit. SYSCON3.10
@IF( ! @DEFINED( __ASC1DIS ) )
	@SET(__ASC1DIS, 0)	; 0 = Disable USART ASC1
@ENDI				; 1 = Enable USART ASC1

	; On-chip I2C Bus Module (I2CDIS) enable bit. SYSCON3.11
@IF( ! @DEFINED( __I2CDIS ) )
	@SET(__I2CDIS, 0 )	; 0 = Disable On-chip I2C Bus Module
@ENDI				; 1 = Enable On-chip I2C Bus Module

	; On-chip Serial Data Link Module (SDLMDIS) enable bit. SYSCON3.12
@IF( ! @DEFINED( __SDLMDIS ) )
	@SET(__SDLMDIS, 0 )	; 0 = Disable On-chip SDL Module 
@ENDI				; 1 = Enable On-chip SDL Module

	; On-chip CAN Module 1 (CAN1DIS) enable bit. SYSCON3.13
@IF( ! @DEFINED( __CAN1DIS ) )
	@SET(__CAN1DIS, 0 )	; 0 = Disable On-chip CAN Module 1
@ENDI				; 1 = Enable On-chip CAN Module 1

	; On-chip CAN Module 2 (CAN2DIS) enable bit. SYSCON3.14
@IF( ! @DEFINED( __CAN2DIS ) )
	@SET(__CAN2DIS, 0 )	; 0 = Disable On-chip CAN Module 2
@ENDI				; 1 = Enable On-chip CAN Module 2

	; Peripheral Clock Driver (PCDDIS) enable bit. SYSCON3.15
@IF( ! @DEFINED( __PCDDIS ) )
	@SET(__PCDDIS, 0 )	; 0 = Disable Peripheral Clock Driver
@ENDI				; 1 = Enable Peripheral Clock Driver

	; Process SYSCON3 values.
	SYSC3	  	EQU	((@__PCDDIS<<15) | (@__CAN2DIS<<14) | (@__CAN1DIS<<13) | (@__SDLMDIS<<12) | (@__I2CDIS<<11) | (@__ASC1DIS<<10) | (@__PWMDIS<<9) | (@__CC6DIS<<8) | (@__CC2DIS<<7) | (@__CC1DIS<<6) |(@__SYSCON3_5<<5) | (@__SYSCON3_4<<4) | (@__GPTDIS<<3) |(@__SSCDIS<<2) |(@__ASC0DIS<<1) | @__ADCDIS)

        ; Enable SYSCON2 and SYSCON3 initialisation.
        ; For newer derivatives, defining the macro SYSCON23 is sufficient
        ; to initialize SYSCON2 and SYSCON3. These next lines are for 
        ; backwards compatibility only.
@IF( @DEFINED( @_CPU ) )      
  @IF( @EQS( @_CPU, "164CI") | @EQS( @_CPU, "164ci") |
       @EQS( @_CPU, "161RI") | @EQS( @_CPU, "161ri") |
       @EQS( @_CPU, "161SI") | @EQS( @_CPU, "161si") |
       @EQS( @_CPU, "161CI") | @EQS( @_CPU, "161ci"))
        @SET(SYSCON23, 1 )	; SYSCON2 and SYSCON3 init enabled
  @ENDI
@ENDI

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. BUSCON0[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC ) )
	@SET(__MCTC, 1)		; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).

	; The Read/Write Signal Delay is 0.5 or 0 State Times. BUSCON0.4
@IF( ! @DEFINED( __RWDC0 ) )
	@SET(__RWDC0, 1)	; 1 = No Delay Time
@ENDI

	; Memory Tri-state is extended by either 1 or 0 State Times. BUSCON0.5
@IF( ! @DEFINED( __MTTC0 ) )
	@SET(__MTTC0, 0)	; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time

	; External bus configurations. BUSCON0[7..6]
					; After reset determined by the state 
					; of the port pins P0L.7 and P0L.6.
					
	; Early Write (EWEN) Enable bit. BUSCON0.8
@IF( ! @DEFINED( __EWEN0 ) )
	@SET(__EWEN0, 0 )	; 0 = Standard write enable signal control
@ENDI				; 1 = Write active state disabled one TCL earlier
	
	; ALE Signal is lengthened by either 1 or 0 State Times. BUSCON0.9
	; Do not disable the ALE lengthening option for a multiplexed bus
	; configuration. See problem 17 in errata sheet SAB-C167A-LM,ES-AC,1.1
	; on page 4/9.

@IF( ! @DEFINED( __ALECTL0 ) )
  @IF( @MUXBUS )
	@SET(__ALECTL0, 1)	; 1 = Delay (Reset value if EA# pin is high
  @ELSE
	@SET(__ALECTL0, 0)	; 0 = No Delay (Reset value if EA# pin is high
  @ENDI
@ENDI

	; Bus Active (BUSACT0) control bit. BUSCON0.10
@IF( ! @DEFINED( __BUSACT0 ) )
	@SET(__BUSACT0, 1 )	; 0 = Disable external bus
@ENDI				; 1 = Enable external bus
	
	; BUSCON Switch (BSWC) Control bit. BUSCON0.11
@IF( ! @DEFINED( __BSWC0 ) )
	@SET(__BSWC0, 0 )	; 0 = Standard bustype switch
@ENDI				; 1 = Wait state included before BUSCON/XBCON
                                ;     switch
	
	; READY# Input Enable control bit. BUSCON0.12
@IF( ! @DEFINED( __RDYEN0 ) )
	@SET(__RDYEN0, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Read Chip Select (CSREN0) enable bit. BUSCON0.14
@IF( ! @DEFINED( __CSREN0 ) )
	@SET(__CSREN0,0)	; 0 = Chip select independend of read command
@ENDI				; 1 = Chip select for duration of read command

	; Write Chip Select (CSWEN0) enable bit. BUSCON0.15
@IF( ! @DEFINED( __CSWEN0 ) )
	@SET(__CSWEN0,0)	; 0 = Chip select independend of write command
@ENDI				; 1 = Chip select for duration of write command

	; Process BUSCON0 low byte and high byte values.
	BUSC0_L		EQU	((@__MTTC0<<5) | (@__RWDC0<<4) | ((~@__MCTC)&000Fh))
	BUSC0_H		EQU	((@__CSWEN0<<7) | (@__CSREN0<<6) | (@__RDYEN0<<4) | (@__BSWC0<<3) | (@__BUSACT0<<2) | (@__ALECTL0<<1) | (@__EWEN0))
	BUSC0_M_L	EQU	03Fh	; Mask low byte BUSCON0
	BUSC0_M_H	EQU	0DFh	; Mask high byte BUSCON0

	; Value definitions for System Configuration Register : BUSCON1

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. BUSCON1[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC1 ) )
	@SET(__MCTC1, 1)	; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).

	; The Read/Write Signal Delay is 0.5 or 0 State Times. BUSCON1.4
@IF( ! @DEFINED( __RWDC1 ) )
	@SET(__RWDC1, 1)	; 1 = No Delay Time
@ENDI				; 0 = Delay Time (Reset value)

	; Memory Tri-state is extended by either 1 or 0 State Times. BUSCON1.5
@IF( ! @DEFINED( __MTTC1 ) )
	@SET(__MTTC1, 0)	; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time

	; External bus configurations. BUSCON1[7..6]
@IF( ! @DEFINED( __BTYP1 ) )
	@SET(__BTYP1, 1)	; 0 = 8-bit Demultiplexed Bus
@ENDI				; 1 = 8-bit Multiplexed Bus
				; 2 = 16-bit Demultiplexed Bus
				; 3 = 16-bit Multiplexed Bus

	; Early Write (EWEN) Enable bit. BUSCON1.8
@IF( ! @DEFINED( __EWEN1 ) )
	@SET(__EWEN1, 0 )	; 0 = Standard write enable signal control
@ENDI				; 1 = Write active state disabled one TCL earlier
	
	; ALE Signal is lengthened by either 1 or 0 State Times. BUSCON1.9
@IF( ! @DEFINED( __ALECTL1 ) )
	@SET(__ALECTL1, 0)	; 0 = No Delay (Reset value if EA# pin is high
@ENDI				; 1 = Delay (Reset value if EA# pin is high

	; Bus Active (BUSACT1) control bit. BUSCON1.10
@IF( ! @DEFINED( __BUSACT1 ) )
	@SET(__BUSACT1, 1 )	; 0 = Disable external bus
@ENDI				; 1 = Enable external bus
	
	; BUSCON Switch (BSWC) Control bit. BUSCON1.11
@IF( ! @DEFINED( __BSWC1 ) )
	@SET(__BSWC1, 0 )	; 0 = Standard bustype switch
@ENDI				; 1 = Wait state included before BUSCON/XBCON
                                ;     switch
	
	; READY# Input Enable control bit. BUSCON1.12
@IF( ! @DEFINED( __RDYEN1 ) )
	@SET(__RDYEN1, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Read Chip Select (CSREN1) enable bit. BUSCON1.14
@IF( ! @DEFINED( __CSREN1 ) )
	@SET(__CSREN1,0)	; 0 = Chip select independend of read command
@ENDI				; 1 = Chip select for duration of read command

	; Write Chip Select (CSWEN1) enable bit. BUSCON1.15
@IF( ! @DEFINED( __CSWEN1 ) )
	@SET(__CSWEN1,0)	; 0 = Chip select independend of write command
@ENDI				; 1 = Chip select for duration of write command

	; Process BUSCON1 values.
	BUSC1		EQU	((@__CSWEN1<<15) | (@__CSREN1<<14) | (@__RDYEN1<<12) | (@__BSWC1<<11) | (@__BUSACT1<<10) | (@__ALECTL1<<9) | (@__EWEN1<<8) | (@__BTYP1<<6) | (@__MTTC1<<5) | (@__RWDC1<<4) | ((~@__MCTC1)&000Fh))

	; Value definitions for System Configuration Register : BUSCON2

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. BUSCON2[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC2 ) )
	@SET(__MCTC2, 1)	; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).

	; The Read/Write Signal Delay is 0.5 or 0 State Times. BUSCON2.4
@IF( ! @DEFINED( __RWDC2 ) )
	@SET(__RWDC2, 1)	; 1 = No Delay Time
@ENDI				; 0 = Delay Time (Reset value)

	; Memory Tri-state is extended by either 1 or 0 State Times. BUSCON2.5
@IF( ! @DEFINED( __MTTC2 ) )
	@SET(__MTTC2, 0)	; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time

	; External bus configurations. BUSCON2[7..6]
@IF( ! @DEFINED( __BTYP2 ) )
	@SET(__BTYP2, 1)	; 0 = 8-bit Demultiplexed Bus
@ENDI				; 1 = 8-bit Multiplexed Bus
				; 2 = 16-bit Demultiplexed Bus
				; 3 = 16-bit Multiplexed Bus

	; Early Write (EWEN) Enable bit. BUSCON2.8
@IF( ! @DEFINED( __EWEN2 ) )
	@SET(__EWEN2, 0 )	; 0 = Standard write enable signal control
@ENDI				; 1 = Write active state disabled one TCL earlier
	
	; ALE Signal is lengthened by either 1 or 0 State Times. BUSCON2.9
@IF( ! @DEFINED( __ALECTL2 ) )
	@SET(__ALECTL2, 0)	; 0 = No Delay (Reset value if EA# pin is high
@ENDI				; 1 = Delay (Reset value if EA# pin is high

	; Bus Active (BUSACT2) control bit. BUSCON2.10
@IF( ! @DEFINED( __BUSACT2 ) )
	@SET(__BUSACT2, 1 )	; 0 = Disable external bus
@ENDI				; 1 = Enable external bus
	
	; BUSCON Switch (BSWC) Control bit. BUSCON2.11
@IF( ! @DEFINED( __BSWC2 ) )
	@SET(__BSWC2, 0 )	; 0 = Standard bustype switch
@ENDI				; 1 = Wait state included before BUSCON/XBCON
                                ;     switch
	
	; READY# Input Enable control bit. BUSCON2.12
@IF( ! @DEFINED( __RDYEN2 ) )
	@SET(__RDYEN2, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Read Chip Select (CSREN2) enable bit. BUSCON2.14
@IF( ! @DEFINED( __CSREN2 ) )
	@SET(__CSREN2,0)	; 0 = Chip select independend of read command
@ENDI				; 1 = Chip select for duration of read command

	; Write Chip Select (CSWEN2) enable bit. BUSCON2.15
@IF( ! @DEFINED( __CSWEN2 ) )
	@SET(__CSWEN2,0)	; 0 = Chip select independend of write command
@ENDI				; 1 = Chip select for duration of write command

	; Process BUSCON2 values
	BUSC2		EQU	((@__CSWEN2<<15) | (@__CSREN2<<14) | (@__RDYEN2<<12) | (@__BSWC2<<11) | (@__BUSACT2<<10) | (@__ALECTL2<<9) | (@__EWEN2<<8) | (@__BTYP2<<6) | (@__MTTC2<<5) | (@__RWDC2<<4) | ((~@__MCTC2)&000Fh))

	; Value definitions for System Configuration Register : BUSCON3

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. BUSCON3[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC3 ) )
	@SET(__MCTC3, 1)	; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).

	; The Read/Write Signal Delay is 0.5 or 0 State Times. BUSCON3.4
@IF( ! @DEFINED( __RWDC3 ) )
	@SET(__RWDC3, 1)	; 1 = No Delay Time
@ENDI				; 0 = Delay Time (Reset value)

	; Memory Tri-state is extended by either 1 or 0 State Times. BUSCON3.5
@IF( ! @DEFINED( __MTTC3 ) )
	@SET(__MTTC3, 0)	; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time

	; External bus configurations. BUSCON3[7..6]
@IF( ! @DEFINED( __BTYP3 ) )
	@SET(__BTYP3, 1)	; 0 = 8-bit Demultiplexed Bus
@ENDI				; 1 = 8-bit Multiplexed Bus
				; 2 = 16-bit Demultiplexed Bus
				; 3 = 16-bit Multiplexed Bus

	; Early Write (EWEN) Enable bit. BUSCON3.8
@IF( ! @DEFINED( __EWEN3 ) )
	@SET(__EWEN3, 0 )	; 0 = Standard write enable signal control
@ENDI				; 1 = Write active state disabled one TCL earlier
	
	; ALE Signal is lengthened by either 1 or 0 State Times. BUSCON3.9
@IF( ! @DEFINED( __ALECTL3 ) )
	@SET(__ALECTL3, 0)	; 0 = No Delay (Reset value if EA# pin is high
@ENDI				; 1 = Delay (Reset value if EA# pin is high

	; Bus Active (BUSACT3) control bit. BUSCON3.10
@IF( ! @DEFINED( __BUSACT3 ) )
	@SET(__BUSACT3, 1 )	; 0 = Disable external bus
@ENDI				; 1 = Enable external bus
	
	; BUSCON Switch (BSWC) Control bit. BUSCON3.11
@IF( ! @DEFINED( __BSWC3 ) )
	@SET(__BSWC3, 0 )	; 0 = Standard bustype switch
@ENDI				; 1 = Wait state included before BUSCON/XBCON
                                ;     switch
	
	; READY# Input Enable control bit. BUSCON3.12
@IF( ! @DEFINED( __RDYEN3 ) )
	@SET(__RDYEN3, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Read Chip Select (CSREN3) enable bit. BUSCON3.14
@IF( ! @DEFINED( __CSREN3 ) )
	@SET(__CSREN3,0)	; 0 = Chip select independend of read command
@ENDI				; 1 = Chip select for duration of read command

	; Write Chip Select (CSWEN3) enable bit. BUSCON3.15
@IF( ! @DEFINED( __CSWEN3 ) )
	@SET(__CSWEN3,0)	; 0 = Chip select independend of write command
@ENDI				; 1 = Chip select for duration of write command

	; Process BUSCON3 values.
	BUSC3		EQU	((@__CSWEN3<<15) | (@__CSREN3<<14) | (@__RDYEN3<<12) | (@__BSWC3<<11) | (@__BUSACT3<<10) | (@__ALECTL3<<9) | (@__EWEN3<<8) | (@__BTYP3<<6) | (@__MTTC3<<5) | (@__RWDC3<<4) | ((~@__MCTC3)&000Fh))

	; Value definitions for System Configuration Register : BUSCON4

	; Memory Cycle Time is extended by a number of additional State Times.
	; in a range from 0 through 15. BUSCON4[3..0]
				; Reset value MCTC = 15 additional state times

@IF( ! @DEFINED( __MCTC4 ) )
	@SET(__MCTC4, 1)	; 1 = Memory wait states is 1 (MCTC = 0EH).
@ENDI				; 0 = Memory wait states is 0 (MCTC = 0FH).

	; The Read/Write Signal Delay is 0.5 or 0 State Times. BUSCON4.4
@IF( ! @DEFINED( __RWDC4 ) )
	@SET(__RWDC4, 1)	; 1 = No Delay Time
@ENDI				; 0 = Delay Time (Reset value)

	; Memory Tri-state is extended by either 1 or 0 State Times. BUSCON4.5
@IF( ! @DEFINED( __MTTC4 ) )
	@SET(__MTTC4, 0)	; 0 = Delay Time (Reset value)
@ENDI				; 1 = No Delay Time

	; External bus configurations. BUSCON4[7..6]
@IF( ! @DEFINED( __BTYP4 ) )
	@SET(__BTYP4, 1)	; 0 = 8-bit Demultiplexed Bus
@ENDI				; 1 = 8-bit Multiplexed Bus
				; 2 = 16-bit Demultiplexed Bus
				; 3 = 16-bit Multiplexed Bus

	; Early Write (EWEN) Enable bit. BUSCON4.8
@IF( ! @DEFINED( __EWEN4 ) )
	@SET(__EWEN4, 0 )	; 0 = Standard write enable signal control
@ENDI				; 1 = Write active state disabled one TCL earlier
	
	; ALE Signal is lengthened by either 1 or 0 State Times. BUSCON4.9
@IF( ! @DEFINED( __ALECTL4 ) )
	@SET(__ALECTL4, 0)	; 0 = No Delay (Reset value if EA# pin is high
@ENDI				; 1 = Delay (Reset value if EA# pin is high

	; Bus Active (BUSACT4) control bit. BUSCON4.10
@IF( ! @DEFINED( __BUSACT4 ) )
	@SET(__BUSACT4, 1 )	; 0 = Disable external bus
@ENDI				; 1 = Enable external bus
	
	; BUSCON Switch (BSWC) Control bit. BUSCON4.11
@IF( ! @DEFINED( __BSWC4 ) )
	@SET(__BSWC4, 0 )	; 0 = Standard bustype switch
@ENDI				; 1 = Wait state included before BUSCON/XBCON
                                ;     switch
	
	; READY# Input Enable control bit. BUSCON4.12
@IF( ! @DEFINED( __RDYEN4 ) )
	@SET(__RDYEN4, 0)	; 0 = Disabled (Reset value)
@ENDI				; 1 = Enabled

	; Read Chip Select (CSREN4) enable bit. BUSCON4.14
@IF( ! @DEFINED( __CSREN4 ) )
	@SET(__CSREN4,0)	; 0 = Chip select independend of read command
@ENDI				; 1 = Chip select for duration of read command

	; Write Chip Select (CSWEN4) enable bit. BUSCON4.15
@IF( ! @DEFINED( __CSWEN4 ) )
	@SET(__CSWEN4,0)	; 0 = Chip select independend of write command
@ENDI				; 1 = Chip select for duration of write command

	; Process BUSCON4 values
	BUSC4		EQU	((@__CSWEN4<<15) | (@__CSREN4<<14) | (@__RDYEN4<<12) | (@__BSWC4<<11) | (@__BUSACT4<<10) | (@__ALECTL4<<9) | (@__EWEN1<<8) | (@__BTYP4<<6) | (@__MTTC4<<5) | (@__RWDC4<<4) | ((~@__MCTC4)&000Fh))

	; Value definitions for System Configuration Register : ADDRSEL1

	; Range Size Selection (RGSZ1) field. ADDRSEL1[0..3]
@IF( ! @DEFINED( __RGSZ1 ) )
	@SET(__RGSZ1, 8 )	; 0 = 4 KB Window
@ENDI				; 1 = 8 KB Window
				; 2 = 16 KB Window
				; 3 = 32 KB Window
				; 4 = 64 KB Window
				; 5 = 128 KB Window
				; 6 = 256 KB Window
				; 7 = 512 KB Window
				; 8 = 1 MB Window
				; 9 = 2 MB Window
				; 10 = 4 MB Window
				; 11 = 8 MB Window

	; Range Start Address (RGSAD1) field. ADDRSEL1[4..15]
@IF( ! @DEFINED( __RGSAD1 ) )
	@SET(__RGSAD1, 100h)	; Defines upper bits of start address
@ENDI

	; Process ADDRSEL1
	ADRS1		EQU	(((@__RGSAD1&0FFFh)<<4) | @__RGSZ1)

	; Value definitions for System Configuration Register : ADDRSEL2

	; Range Size Selection (RGSZ2) field. ADDRSEL2[0..3]
@IF( ! @DEFINED( __RGSZ2 ) )
	@SET(__RGSZ2, 8 )	; 0 = 4 KB Window
@ENDI				; 1 = 8 KB Window
				; 2 = 16 KB Window
				; 3 = 32 KB Window
				; 4 = 64 KB Window
				; 5 = 128 KB Window
				; 6 = 256 KB Window
				; 7 = 512 KB Window
				; 8 = 1 MB Window
				; 9 = 2 MB Window
				; 10 = 4 MB Window
				; 11 = 8 MB Window

	; Range Start Address (RGSAD2) field. ADDRSEL2[4..15]
@IF( ! @DEFINED( __RGSAD2 ) )
	@SET(__RGSAD2, 200H)	; Defines upper bits of start address
@ENDI

	; Process ADDRSEL2
	ADRS2		EQU	(((@__RGSAD2&0FFFh)<<4) | @__RGSZ2)

	; Value definitions for System Configuration Register : ADDRSEL3

	; Range Size Selection (RGSZ3) field. ADDRSEL3[0..3]
@IF( ! @DEFINED( __RGSZ3 ) )
	@SET(__RGSZ3, 8 )	; 0 = 4 KB Window
@ENDI				; 1 = 8 KB Window
				; 2 = 16 KB Window
				; 3 = 32 KB Window
				; 4 = 64 KB Window
				; 5 = 128 KB Window
				; 6 = 256 KB Window
				; 7 = 512 KB Window
				; 8 = 1 MB Window
				; 9 = 2 MB Window
				; 10 = 4 MB Window
				; 11 = 8 MB Window

	; Range Start Address (RGSAD3) field. ADDRSEL3[4..15]
@IF( ! @DEFINED( __RGSAD3 ) )
	@SET(__RGSAD3, 300H)	; Defines upper bits of start address
@ENDI

	; Process ADDRSEL3
	ADRS3		EQU	(((@__RGSAD3&0FFFh)<<4) | @__RGSZ3)

	; Value definitions for System Configuration Register : ADDRSEL4

	; Range Size Selection (RGSZ4) field. ADDRSEL4[0..3]
@IF( ! @DEFINED( __RGSZ4 ) )
	@SET(__RGSZ4, 8 )	; 0 = 4 KB Window
@ENDI				; 1 = 8 KB Window
				; 2 = 16 KB Window
				; 3 = 32 KB Window
				; 4 = 64 KB Window
				; 5 = 128 KB Window
				; 6 = 256 KB Window
				; 7 = 512 KB Window
				; 8 = 1 MB Window
				; 9 = 2 MB Window
				; 10 = 4 MB Window
				; 11 = 8 MB Window

	; Range Start Address (RGSAD4) field. ADDRSEL4[4..15]
@IF( ! @DEFINED( __RGSAD4 ) )
	@SET(__RGSAD4, 400H)	; Defines upper bits of start address
@ENDI

	; Process ADDRSEL4
	ADRS4		EQU	(((@__RGSAD4&0FFFh)<<4) | @__RGSZ4)

	; Value definitions for System Configuration Register : XPERCON

	; X-Peripheral enable bit. XPERCON.0
@IF( ! @DEFINED( __XPER0 ) )
	@SET(__XPER0, 1)	; 0 = Disable X-Peripheral 0
				; 1 = Enable X-Peripheral 0 (reset)
@ENDI

	; X-Peripheral enable bit. XPERCON.1
@IF( ! @DEFINED( __XPER1 ) )
	@SET(__XPER1, 0)	; 0 = Disable X-Peripheral 1 (reset)
				; 1 = Enable X-Peripheral 1
@ENDI

	; X-Peripheral enable bit. XPERCON.10
@IF( ! @DEFINED( __XPER10 ) )
	@SET(__XPER10, 1)	; 0 = Disable X-Peripheral 10
				; 1 = Enable X-Peripheral 10 (reset)
@ENDI

	; X-Peripheral enable bit. XPERCON.11
@IF( ! @DEFINED( __XPER11 ) )
	@SET(__XPER11, 0)	; 0 = Disable X-Peripheral 11 (reset)
				; 1 = Enable X-Peripheral 11
@ENDI

	; X-Peripheral enable bit. XPERCON.14
@IF( ! @DEFINED( __XPER14 ) )
	@SET(__XPER14, 0)	; 0 = Disable X-Peripheral 14 (reset)
				; 1 = Enable X-Peripheral 14
@ENDI

	; Process XPERCON
	XPERC		EQU	((@__XPER14<<14) | (@__XPER11<<11) | (@__XPER10<<10) | (@__XPER1<<1) | (@__XPER0))

@IF( @EVA )
	; Symbols __EVA_SYSCON and __EVA_MEMSZ should be defined when the 
	; XVW166 debugger uses the EVA167 target board as its execution
	; environment.
	; They let the debugger configure the target environment correctly 
	; before the boot program downloads the monitor program MON167.
	; 
	__MEMSZ		LIT	'0'	; EVA167 Memory configuration
					; 0 =  1 MBit RAM (Reset value)
					; 1 =  2 MBit RAM
					; 2 =  4 MBit RAM

	__EVA_SYSCON	EQU	((SYSC_H << 8) | SYSC_L)
	__EVA_MEMSZ	EQU	__MEMSZ
	PUBLIC	__EVA_SYSCON,__EVA_MEMSZ
@ENDI

;*****************************************************************************
;* __CSTART
;*****************************************************************************
__CSTART_PR	SECTION CODE WORD PUBLIC 'CPROGRAM'
__CSTART 	PROC TASK __CSTART_TASK INTNO __CSTART_INUM = 00H

	DISWDT				; Disable watchdog timer.

  @IF( @DEFINED( @XPERCON ) )
	EXTR	#01H				; Extended SFR access.
	MOV     XPERCON, #XPERC			; set XPERCON register.
  @ENDI

						; Set SYSCON register.
	BFLDL	SYSCON, #SYSC_M_L, #(SYSC_L AND SYSC_M_L)
	BFLDH	SYSCON, #SYSC_M_H, #(SYSC_H AND SYSC_M_H)

  @IF( @DEFINED( @SYSCON1 ) )
	EXTR	#01H				; Extended SFR access.
	MOV	SYSCON1, #SYSC1			; Set SYSCON1 register.
  @ENDI

  @IF( @DEFINED( @SYSCON23 ) )
	EXTR	#03H				; Extended SFR access.
						; Set SYSCON2 register.
	BFLDL	SYSCON2, #SYSC2_M_L, #(SYSC2_L AND SYSC2_M_L)
	BFLDH	SYSCON2, #SYSC2_M_H, #(SYSC2_H AND SYSC2_M_H)
	MOV	SYSCON3, #SYSC3			; Set SYSCON3 register.
  @ENDI

						; Set BUSCON0 register.
	BFLDL	BUSCON0, #BUSC0_M_L, #(BUSC0_L AND BUSC0_M_L)
	BFLDH	BUSCON0, #BUSC0_M_H, #(BUSC0_H AND BUSC0_M_H)

  @IF( @DEFINED( @BUSCON1 ) )
	MOV	ADDRSEL1, #ADRS1		; Set ADDRSEL1 register.
	MOV	BUSCON1, #BUSC1			; Set BUSCON1 register.
  @ENDI
  @IF( @DEFINED( @BUSCON2 ) )
	MOV	ADDRSEL2, #ADRS2		; Set ADDRSEL2 register.
	MOV	BUSCON2, #BUSC2			; Set BUSCON2 register.
  @ENDI
  @IF( @DEFINED( @BUSCON3 ) )
	MOV	ADDRSEL3, #ADRS3		; Set ADDRSEL3 register.
	MOV	BUSCON3, #BUSC3			; Set BUSCON3 register.
  @ENDI
  @IF( @DEFINED( @BUSCON4 ) )
	MOV	ADDRSEL4, #ADRS4		; Set ADDRSEL4 register.
	MOV	BUSCON4, #BUSC4			; Set BUSCON4 register.
  @ENDI

	MOV	STKOV,	#?SYSSTACK_BOTTOM + 6*2	; Set stack underflow pointer.
	MOV	STKUN,	#?SYSSTACK_TOP		; Set stack overflow pointer.
	MOV	SP,	#?SYSSTACK_TOP		; Set stack pointer.

	MOV	CP, 	#RBANK		; Set context pointer.
	NOP			
	MOV	R0, #BOTTOM_BITRAM	; Set R0	

@IF( @DEFINED( @CALLEINIT))
@_CALL( @CALLEINIT, R1)		
@ENDI

	EINIT				; End of initialization

; start of init function
@_CALL(__C_INIT, R1)		
; end of init function

  @IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
	MOV	R0, #POF ?USRSTACK_TOP	; set user stack pointer
	BSET	R0.0EH			; User stack uses DPP1
  @ELSE
	MOV	R0, #?USRSTACK_TOP	; set user stack pointer
  @ENDI

  @IF( @EVA )
        BSET    IEN                     ; allow monitor to break application
  @ENDI
 
        MOV     R12, #0                 ; set argc to 0
        MOV     R13, #0                 ;
        MOV     R14, #0                 ; set argv[] to 0

@IF( @DEFINED( @CALLINIT))
@_CALL( @CALLINIT, R1)		
@ENDI

@_CALL( _main, R1)
								
; The exit() function causes normal program termination to occur. First, all 
; functions registered by the atexit() function are called in the reverse 
; order. Next, all open streams with unwritten buffered data are flushed, all 
; open streams are closed and all files created by the tmpfile() function are 
; removed. The status value passed to exit is returned in R4.
__EXIT: LABEL FAR			; the exit() or abort() function jumps
					; to this entry.
@IF( @EX_AB )
					; restore (host) environment.
	MOV	SP, #?SYSSTACK_TOP	; restore stack pointer.
  @IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
	MOV	R0, #POF ?USRSTACK_TOP	; restore user stack pointer
	BSET	R0.0EH			; User stack uses DPP1
  @ELSE
	MOV	R0, #?USRSTACK_TOP	; restore user stack pointer
  @ENDI
  @IF( @FLOAT )
	MOV	R1, #?FPSTKUN		; restore floating point stack pointer.
	MOV	?FPSP, R1		; ?FPSP normally initialized via 
					; C166_INIT.
  @ENDI	
@ENDI

__IDLE: IDLE				; Power down CPU until peripheral inter-
					; rupt or external interrupt occurs.
	JMPR	CC_UC, __IDLE		; set idle mode again.
	RETV				; Virtual return.
__CSTART	ENDP
__CSTART_PR	ENDS

@IF( @EQS(@MODEL,"TINY") | @EQS(@MODEL,"SMALL") )
C166_US	SECTION	LDAT WORD GLBUSRSTACK 'CUSTACK'
@ENDI
@IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
C166_US	SECTION	DATA WORD GLBUSRSTACK 'CUSTACK'
@ENDI
@IF( @_USRSTACK & ( @EQS(@MODEL,"SMALL") | @EQS(@MODEL,"LARGE") ) )
  @IF( @EQS( @MODEL, "SMALL" ) | @EQS( @MODEL, "LARGE" ) )
	DS	2+4		; Allocate a user stack of at least 2 bytes +
				; return label main (__MAIN_RET).
  @ELSE
	DS	2+2		; Allocate a user stack of at least 2 bytes +
				; return label main (__MAIN_RET).
  @ENDI
@ELSE
	DS	2		; Allocate a user stack of at least 2 bytes
@ENDI
C166_US	ENDS

@IF( @EQS(@MODEL,"MEDIUM") | @EQS(@MODEL,"LARGE") )
C166_DGROUP	DGROUP	__DUMMY
__DUMMY	SECTION DATA WORD PUBLIC 'CNEAR'
__DUMMY	ENDS
@ENDI

RBANK	REGDEF R0-R15			; Register usage
	SSKDEF	@__STKSZ		; System stack size
	
	END
