#ifndef REG167CR_INC
#define REG167CR_INC


/* @(#)reg167cr.h	1.1 98/07/24 */

/************************************************************************/
/*                                                                      */
/* Copyright (c) 1998 TASKING Inc.                                      */
/*                                                                      */
/* This SFR header file can be used with c166.                          */
/* It contains all (E)SFR names and BIT names and on-chip               */
/* register definitions for the SAB C167CR, based on                    */
/* the "C167 User's Manual 03.96 Version 2.0" and                       */
/* the "C167CR Advance Product Information 06.95" and                   */
/* the "C167CR-16F Advance Product Information 06.95" and               */
/* the "C167CR / C167SR Addendum to C167 User's Manual M14T180" and     */
/* the "C167CR-4RM Data Sheet 12.96 (Advance Information)" and          */
/* the "C167CR-16RM Data Sheet 12.96 (Advance Information)" and         */
/* the "C167CR-16F Data Sheet 03.97 Advance Information" and            */
/* the "C167CR/C167SR-LC Data Sheet 05.97 (Bare Die Delivery)" and      */
/* the "C167SR/CR-L25M Data Sheet Addendum 1998-03 Advance Information".*/
/*                                                                      */
/* The file is divided into several parts which are all                 */
/* enabled by default. If one of these parts is not                     */
/* needed, the appropriate macro should be defined                      */
/* in your program before including this file.                          */
/*                                                                      */
/* The following 'steering' macros can be used:                         */
/*                                                                      */
/*      REG167CR_NOCPU       omit cpu registers                         */
/*      REG167CR_NOPEC       omit PEC registers                         */
/*      REG167CR_NOADINT     omit additional interrupt registers        */
/*      REG167CR_NOPORT      omit port I/O registers                    */
/*      REG167CR_NOTIMER     omit timer registers                       */
/*      REG167CR_NOCAPCOM    omit CAPCOM registers                      */
/*      REG167CR_NORS232     omit serial I/O registers                  */
/*      REG167CR_NOPWM       omit PWM registers                         */
/*      REG167CR_NOADC       omit A/D Converter registers               */
/*      REG167CR_NOXPER      omit X-Peripheral registers                */
/*                                                                      */
/**********************************************************************************/

#ifndef REG167CR_NOCPU
/* Renaming of old MON and VIS bits in fixed register set for SYSCON    */
#define XPERSHARE		MON
#define VISIBLE		VIS
#endif /* REG167CR_NOCPU */

#ifndef REG167CR_NOCPU
sfr SYSCON;
sfrbit MON;		/* SYSCON.0 */
sfrbit VIS;		/* SYSCON.1 */
sfrbit XPEN;		/* SYSCON.2 */
sfrbit WRCFG;		/* SYSCON.7 */
sfrbit CLKEN;		/* SYSCON.8 */
sfrbit BYTDIS;		/* SYSCON.9 */
sfrbit ROMEN;		/* SYSCON.10 */
sfrbit SGTDIS;		/* SYSCON.11 */
sfrbit ROMS1;		/* SYSCON.12 */

sfr PSW;
sfrbit N;		/* PSW.0 */
sfrbit C;		/* PSW.1 */
sfrbit V;		/* PSW.2 */
sfrbit Z;		/* PSW.3 */
sfrbit E;		/* PSW.4 */
sfrbit MULIP;		/* PSW.5 */
sfrbit USR0;		/* PSW.6 */
sfrbit HLDEN;		/* PSW.10 */
sfrbit IEN;		/* PSW.11 */

sfr CSP;
sfr DPP0;
sfr DPP1;
sfr DPP2;
sfr DPP3;
sfr CP;
sfr SP;
sfr STKOV;
sfr STKUN;
sfr MDH;
sfr MDL;

sfr MDC;
sfrbit MDRIU;		/* MDC.4 */

sfr ZEROS;
sfr ONES;
#endif /* REG167CR_NOCPU */

#ifndef REG167CR_NOPEC
sfr PECC0;
sfr PECC1;
sfr PECC2;
sfr PECC3;
sfr PECC4;
sfr PECC5;
sfr PECC6;
sfr PECC7;
sfr SRCP0;
sfr DSTP0;
sfr SRCP1;
sfr DSTP1;
sfr SRCP2;
sfr DSTP2;
sfr SRCP3;
sfr DSTP3;
sfr SRCP4;
sfr DSTP4;
sfr SRCP5;
sfr DSTP5;
sfr SRCP6;
sfr DSTP6;
sfr SRCP7;
sfr DSTP7;
#endif /* REG167CR_NOPEC */

#ifndef REG167CR_NOADINT
esfr EXICON;
#endif /* REG167CR_NOADINT */

#ifndef REG167CR_NOCPU
sfr TFR;
sfrbit ILLBUS;		/* TFR.0 */
sfrbit ILLINA;		/* TFR.1 */
sfrbit ILLOPA;		/* TFR.2 */
sfrbit PRTFLT;		/* TFR.3 */
sfrbit UNDOPC;		/* TFR.7 */
sfrbit STKUF;		/* TFR.13 */
sfrbit STKOF;		/* TFR.14 */
sfrbit NMI;		/* TFR.15 */
#endif /* REG167CR_NOCPU */

#ifndef REG167CR_NOPORT
esfr PICON;
esfrbit P2LIN;		/* PICON.0 */
esfrbit P2HIN;		/* PICON.1 */
esfrbit P3LIN;		/* PICON.2 */
esfrbit P3HIN;		/* PICON.3 */
esfrbit P7LIN;		/* PICON.6 */
esfrbit P8LIN;		/* PICON.7 */

sfr P0L;
sfrbit D0;		/* P0L.0 */
sfrbit D1;		/* P0L.1 */
sfrbit D2;		/* P0L.2 */
sfrbit D3;		/* P0L.3 */
sfrbit D4;		/* P0L.4 */
sfrbit D5;		/* P0L.5 */
sfrbit D6;		/* P0L.6 */
sfrbit D7;		/* P0L.7 */
#define AD0		D0
#define AD1		D1
#define AD2		D2
#define AD3		D3
#define AD4		D4
#define AD5		D5
#define AD6		D6
#define AD7		D7

sfr P0H;
sfrbit D8;		/* P0H.0 */
sfrbit D9;		/* P0H.1 */
sfrbit D10;		/* P0H.2 */
sfrbit D11;		/* P0H.3 */
sfrbit D12;		/* P0H.4 */
sfrbit D13;		/* P0H.5 */
sfrbit D14;		/* P0H.6 */
sfrbit D15;		/* P0H.7 */
#define AD8		D8
#define AD9		D9
#define AD10		D10
#define AD11		D11
#define AD12		D12
#define AD13		D13
#define AD14		D14
#define AD15		D15

esfr DP0L;
esfr DP0H;

sfr P1L;
sfrbit A0;		/* P1L.0 */
sfrbit A1;		/* P1L.1 */
sfrbit A2;		/* P1L.2 */
sfrbit A3;		/* P1L.3 */
sfrbit A4;		/* P1L.4 */
sfrbit A5;		/* P1L.5 */
sfrbit A6;		/* P1L.6 */
sfrbit A7;		/* P1L.7 */

sfr P1H;
sfrbit CC24IO;		/* P1H.4 */
sfrbit CC25IO;		/* P1H.5 */
sfrbit CC26IO;		/* P1H.6 */
sfrbit CC27IO;		/* P1H.7 */

esfr DP1L;
esfr DP1H;

sfr P2;
sfrbit CC0IO;		/* P2.0 */
sfrbit CC1IO;		/* P2.1 */
sfrbit CC2IO;		/* P2.2 */
sfrbit CC3IO;		/* P2.3 */
sfrbit CC4IO;		/* P2.4 */
sfrbit CC5IO;		/* P2.5 */
sfrbit CC6IO;		/* P2.6 */
sfrbit CC7IO;		/* P2.7 */
sfrbit CC8IO;		/* P2.8 */
sfrbit CC9IO;		/* P2.9 */
sfrbit CC10IO;		/* P2.10 */
sfrbit CC11IO;		/* P2.11 */
sfrbit CC12IO;		/* P2.12 */
sfrbit CC13IO;		/* P2.13 */
sfrbit CC14IO;		/* P2.14 */
sfrbit CC15IO;		/* P2.15 */
#define EX0IN		CC8IO
#define EX1IN		CC9IO
#define EX2IN		CC10IO
#define EX3IN		CC11IO
#define EX4IN		CC12IO
#define EX5IN		CC13IO
#define EX6IN		CC14IO
#define EX7IN		CC15IO
#define T7IN		CC15IO

sfr DP2;
esfr ODP2;

sfr P3;
sfrbit T0IN;		/* P3.0 */
sfrbit T6OUT;		/* P3.1 */
sfrbit CAPIN;		/* P3.2 */
sfrbit T3OUT;		/* P3.3 */
sfrbit T3EUD;		/* P3.4 */
sfrbit T4IN;		/* P3.5 */
sfrbit T3IN;		/* P3.6 */
sfrbit T2IN;		/* P3.7 */
sfrbit MRST;		/* P3.8 */
sfrbit MTSR;		/* P3.9 */
sfrbit TXD0;		/* P3.10 */
sfrbit RXD0;		/* P3.11 */
sfrbit BHE;		/* P3.12 */
sfrbit SCLK;		/* P3.13 */
sfrbit CLKOUT;		/* P3.15 */
#define WRH		BHE
/* Backwards compatibility values                                       */
#define SSCLK		SCLK
#define SSDI		MRST
#define SSDO		MTSR

sfr DP3;
esfr ODP3;

sfr P4;
sfrbit A16;		/* P4.0 */
sfrbit A17;		/* P4.1 */
sfrbit A18;		/* P4.2 */
sfrbit A19;		/* P4.3 */
sfrbit A20;		/* P4.4 */
sfrbit A21;		/* P4.5 */
sfrbit A22;		/* P4.6 */
sfrbit A23;		/* P4.7 */

sfr DP4;

sfr P5;
sfrbit AN0;		/* P5.0 */
sfrbit AN1;		/* P5.1 */
sfrbit AN2;		/* P5.2 */
sfrbit AN3;		/* P5.3 */
sfrbit AN4;		/* P5.4 */
sfrbit AN5;		/* P5.5 */
sfrbit AN6;		/* P5.6 */
sfrbit AN7;		/* P5.7 */
sfrbit AN8;		/* P5.8 */
sfrbit AN9;		/* P5.9 */
sfrbit AN10;		/* P5.10 */
sfrbit AN11;		/* P5.11 */
sfrbit AN12;		/* P5.12 */
sfrbit AN13;		/* P5.13 */
sfrbit AN14;		/* P5.14 */
sfrbit AN15;		/* P5.15 */
#define T6EUD		AN10
#define T5EUD		AN11
#define T6IN		AN12
#define T5IN		AN13
#define T4EUD		AN14
#define T2EUD		AN15

sfr P6;
sfrbit CS0;		/* P6.0 */
sfrbit CS1;		/* P6.1 */
sfrbit CS2;		/* P6.2 */
sfrbit CS3;		/* P6.3 */
sfrbit CS4;		/* P6.4 */
sfrbit HOLD;		/* P6.5 */
sfrbit HLDA;		/* P6.6 */
sfrbit BREQ;		/* P6.7 */

sfr DP6;
esfr ODP6;

sfr P7;
sfrbit POUT0;		/* P7.0 */
sfrbit POUT1;		/* P7.1 */
sfrbit POUT2;		/* P7.2 */
sfrbit POUT3;		/* P7.3 */
sfrbit CC28IO;		/* P7.4 */
sfrbit CC29IO;		/* P7.5 */
sfrbit CC30IO;		/* P7.6 */
sfrbit CC31IO;		/* P7.7 */

sfr DP7;
esfr ODP7;

sfr P8;
sfrbit CC16IO;		/* P8.0 */
sfrbit CC17IO;		/* P8.1 */
sfrbit CC18IO;		/* P8.2 */
sfrbit CC19IO;		/* P8.3 */
sfrbit CC20IO;		/* P8.4 */
sfrbit CC21IO;		/* P8.5 */
sfrbit CC22IO;		/* P8.6 */
sfrbit CC23IO;		/* P8.7 */

sfr DP8;
esfr ODP8;
#endif /* REG167CR_NOPORT */

#ifndef REG167CR_NOCPU
sfr BUSCON0;
sfrbit RWDC0;		/* BUSCON0.4 */
sfrbit MTTC0;		/* BUSCON0.5 */
sfrbit ALECTL0;		/* BUSCON0.9 */
sfrbit BUSACT0;		/* BUSCON0.10 */
sfrbit RDYEN0;		/* BUSCON0.12 */
sfrbit CSREN0;		/* BUSCON0.14 */
sfrbit CSWEN0;		/* BUSCON0.15 */

sfr BUSCON1;
sfrbit RWDC1;		/* BUSCON1.4 */
sfrbit MTTC1;		/* BUSCON1.5 */
sfrbit ALECTL1;		/* BUSCON1.9 */
sfrbit BUSACT1;		/* BUSCON1.10 */
sfrbit RDYEN1;		/* BUSCON1.12 */
sfrbit CSREN1;		/* BUSCON1.14 */
sfrbit CSWEN1;		/* BUSCON1.15 */

sfr BUSCON2;
sfrbit RWDC2;		/* BUSCON2.4 */
sfrbit MTTC2;		/* BUSCON2.5 */
sfrbit ALECTL2;		/* BUSCON2.9 */
sfrbit BUSACT2;		/* BUSCON2.10 */
sfrbit RDYEN2;		/* BUSCON2.12 */
sfrbit CSREN2;		/* BUSCON2.14 */
sfrbit CSWEN2;		/* BUSCON2.15 */

sfr BUSCON3;
sfrbit RWDC3;		/* BUSCON3.4 */
sfrbit MTTC3;		/* BUSCON3.5 */
sfrbit ALECTL3;		/* BUSCON3.9 */
sfrbit BUSACT3;		/* BUSCON3.10 */
sfrbit RDYEN3;		/* BUSCON3.12 */
sfrbit CSREN3;		/* BUSCON3.14 */
sfrbit CSWEN3;		/* BUSCON3.15 */

sfr BUSCON4;
sfrbit RWDC4;		/* BUSCON4.4 */
sfrbit MTTC4;		/* BUSCON4.5 */
sfrbit ALECTL4;		/* BUSCON4.9 */
sfrbit BUSACT4;		/* BUSCON4.10 */
sfrbit RDYEN4;		/* BUSCON4.12 */
sfrbit CSREN4;		/* BUSCON4.14 */
sfrbit CSWEN4;		/* BUSCON4.15 */

sfr ADDRSEL1;
sfr ADDRSEL2;
sfr ADDRSEL3;
sfr ADDRSEL4;
#endif /* REG167CR_NOCPU */

#ifndef REG167CR_NOPORT
esfr RP0H;
#endif /* REG167CR_NOPORT */

#ifndef REG167CR_NOTIMER
sfr T3CON;
sfrbit T3R;		/* T3CON.6 */
sfrbit T3UD;		/* T3CON.7 */
sfrbit T3UDE;		/* T3CON.8 */
sfrbit T3OE;		/* T3CON.9 */
sfrbit T3OTL;		/* T3CON.10 */

sfr T2CON;
sfrbit T2R;		/* T2CON.6 */
sfrbit T2UD;		/* T2CON.7 */
sfrbit T2UDE;		/* T2CON.8 */

sfr T4CON;
sfrbit T4R;		/* T4CON.6 */
sfrbit T4UD;		/* T4CON.7 */
sfrbit T4UDE;		/* T4CON.8 */

sfr T2IC;
sfrbit T2IE;		/* T2IC.6 */
sfrbit T2IR;		/* T2IC.7 */

sfr T3IC;
sfrbit T3IE;		/* T3IC.6 */
sfrbit T3IR;		/* T3IC.7 */

sfr T4IC;
sfrbit T4IE;		/* T4IC.6 */
sfrbit T4IR;		/* T4IC.7 */

sfr T6CON;
sfrbit T6R;		/* T6CON.6 */
sfrbit T6UD;		/* T6CON.7 */
sfrbit T6UDE;		/* T6CON.8 */
sfrbit T6OE;		/* T6CON.9 */
sfrbit T6OTL;		/* T6CON.10 */
sfrbit T6SR;		/* T6CON.15 */

sfr T5CON;
sfrbit T5R;		/* T5CON.6 */
sfrbit T5UD;		/* T5CON.7 */
sfrbit T5UDE;		/* T5CON.8 */
sfrbit T5CLR;		/* T5CON.14 */
sfrbit T5SC;		/* T5CON.15 */

sfr T5IC;
sfrbit T5IE;		/* T5IC.6 */
sfrbit T5IR;		/* T5IC.7 */

sfr T6IC;
sfrbit T6IE;		/* T6IC.6 */
sfrbit T6IR;		/* T6IC.7 */

sfr CRIC;
sfrbit CRIE;		/* CRIC.6 */
sfrbit CRIR;		/* CRIC.7 */
#endif /* REG167CR_NOTIMER */

#ifndef REG167CR_NORS232
sfr S0CON;
sfrbit S0STP;		/* S0CON.3 */
sfrbit S0REN;		/* S0CON.4 */
sfrbit S0PEN;		/* S0CON.5 */
sfrbit S0FEN;		/* S0CON.6 */
sfrbit S0OEN;		/* S0CON.7 */
sfrbit S0PE;		/* S0CON.8 */
sfrbit S0FE;		/* S0CON.9 */
sfrbit S0OE;		/* S0CON.10 */
sfrbit S0ODD;		/* S0CON.12 */
sfrbit S0BRS;		/* S0CON.13 */
sfrbit S0LB;		/* S0CON.14 */
sfrbit S0R;		/* S0CON.15 */

sfr S0TIC;
sfrbit S0TIE;		/* S0TIC.6 */
sfrbit S0TIR;		/* S0TIC.7 */

sfr S0RIC;
sfrbit S0RIE;		/* S0RIC.6 */
sfrbit S0RIR;		/* S0RIC.7 */

sfr S0EIC;
sfrbit S0EIE;		/* S0EIC.6 */
sfrbit S0EIR;		/* S0EIC.7 */

esfr S0TBIC;
esfrbit S0TBIE;		/* S0TBIC.6 */
esfrbit S0TBIR;		/* S0TBIC.7 */

sfr SSCCON;
sfrbit SSCHB;		/* SSCCON.4 */
sfrbit SSCPH;		/* SSCCON.5 */
sfrbit SSCPO;		/* SSCCON.6 */
sfrbit SSCTEN;		/* SSCCON.8 */
sfrbit SSCREN;		/* SSCCON.9 */
sfrbit SSCPEN;		/* SSCCON.10 */
sfrbit SSCBEN;		/* SSCCON.11 */
sfrbit SSCAREN;		/* SSCCON.12 */
sfrbit SSCMS;		/* SSCCON.14 */
sfrbit SSCEN;		/* SSCCON.15 */
#define SSCTE		SSCTEN
#define SSCRE		SSCREN
#define SSCPE		SSCPEN
#define SSCBE		SSCBEN
#define SSCBSY		SSCAREN

sfr SSCTIC;
sfrbit SSCTIE;		/* SSCTIC.6 */
sfrbit SSCTIR;		/* SSCTIC.7 */

sfr SSCRIC;
sfrbit SSCRIE;		/* SSCRIC.6 */
sfrbit SSCRIR;		/* SSCRIC.7 */

sfr SSCEIC;
sfrbit SSCEIE;		/* SSCEIC.6 */
sfrbit SSCEIR;		/* SSCEIC.7 */

sfr S0BG;
sfr S0RBUF;
sfr S0TBUF;
esfr SSCBR;
esfr SSCRB;
esfr SSCTB;
#endif /* REG167CR_NORS232 */

#ifndef REG167CR_NOTIMER
sfr WDTCON;
sfrbit WDTIN;		/* WDTCON.0 */
sfrbit WDTR;		/* WDTCON.1 */

sfr WDT;
#endif /* REG167CR_NOTIMER */

#ifndef REG167CR_NOCAPCOM
sfr T01CON;
sfrbit T0M;		/* T01CON.3 */
sfrbit T0R;		/* T01CON.6 */
sfrbit T1M;		/* T01CON.11 */
sfrbit T1R;		/* T01CON.14 */

sfr T78CON;
sfrbit T7M;		/* T78CON.3 */
sfrbit T7R;		/* T78CON.6 */
sfrbit T8M;		/* T78CON.11 */
sfrbit T8R;		/* T78CON.14 */

sfr T0IC;
sfrbit T0IE;		/* T0IC.6 */
sfrbit T0IR;		/* T0IC.7 */

sfr T1IC;
sfrbit T1IE;		/* T1IC.6 */
sfrbit T1IR;		/* T1IC.7 */

esfr T7IC;
esfrbit T7IE;		/* T7IC.6 */
esfrbit T7IR;		/* T7IC.7 */

esfr T8IC;
esfrbit T8IE;		/* T8IC.6 */
esfrbit T8IR;		/* T8IC.7 */

sfr CCM0;
sfrbit ACC0;		/* CCM0.3 */
sfrbit ACC1;		/* CCM0.7 */
sfrbit ACC2;		/* CCM0.11 */
sfrbit ACC3;		/* CCM0.15 */

sfr CCM1;
sfrbit ACC4;		/* CCM1.3 */
sfrbit ACC5;		/* CCM1.7 */
sfrbit ACC6;		/* CCM1.11 */
sfrbit ACC7;		/* CCM1.15 */

sfr CCM2;
sfrbit ACC8;		/* CCM2.3 */
sfrbit ACC9;		/* CCM2.7 */
sfrbit ACC10;		/* CCM2.11 */
sfrbit ACC11;		/* CCM2.15 */

sfr CCM3;
sfrbit ACC12;		/* CCM3.3 */
sfrbit ACC13;		/* CCM3.7 */
sfrbit ACC14;		/* CCM3.11 */
sfrbit ACC15;		/* CCM3.15 */

sfr CCM4;
sfrbit ACC16;		/* CCM4.3 */
sfrbit ACC17;		/* CCM4.7 */
sfrbit ACC18;		/* CCM4.11 */
sfrbit ACC19;		/* CCM4.15 */

sfr CCM5;
sfrbit ACC20;		/* CCM5.3 */
sfrbit ACC21;		/* CCM5.7 */
sfrbit ACC22;		/* CCM5.11 */
sfrbit ACC23;		/* CCM5.15 */

sfr CCM6;
sfrbit ACC24;		/* CCM6.3 */
sfrbit ACC25;		/* CCM6.7 */
sfrbit ACC26;		/* CCM6.11 */
sfrbit ACC27;		/* CCM6.15 */

sfr CCM7;
sfrbit ACC28;		/* CCM7.3 */
sfrbit ACC29;		/* CCM7.7 */
sfrbit ACC30;		/* CCM7.11 */
sfrbit ACC31;		/* CCM7.15 */

sfr CC0IC;
sfrbit CC0IE;		/* CC0IC.6 */
sfrbit CC0IR;		/* CC0IC.7 */

sfr CC1IC;
sfrbit CC1IE;		/* CC1IC.6 */
sfrbit CC1IR;		/* CC1IC.7 */

sfr CC2IC;
sfrbit CC2IE;		/* CC2IC.6 */
sfrbit CC2IR;		/* CC2IC.7 */

sfr CC3IC;
sfrbit CC3IE;		/* CC3IC.6 */
sfrbit CC3IR;		/* CC3IC.7 */

sfr CC4IC;
sfrbit CC4IE;		/* CC4IC.6 */
sfrbit CC4IR;		/* CC4IC.7 */

sfr CC5IC;
sfrbit CC5IE;		/* CC5IC.6 */
sfrbit CC5IR;		/* CC5IC.7 */

sfr CC6IC;
sfrbit CC6IE;		/* CC6IC.6 */
sfrbit CC6IR;		/* CC6IC.7 */

sfr CC7IC;
sfrbit CC7IE;		/* CC7IC.6 */
sfrbit CC7IR;		/* CC7IC.7 */

sfr CC8IC;
sfrbit CC8IE;		/* CC8IC.6 */
sfrbit CC8IR;		/* CC8IC.7 */

sfr CC9IC;
sfrbit CC9IE;		/* CC9IC.6 */
sfrbit CC9IR;		/* CC9IC.7 */

sfr CC10IC;
sfrbit CC10IE;		/* CC10IC.6 */
sfrbit CC10IR;		/* CC10IC.7 */

sfr CC11IC;
sfrbit CC11IE;		/* CC11IC.6 */
sfrbit CC11IR;		/* CC11IC.7 */

sfr CC12IC;
sfrbit CC12IE;		/* CC12IC.6 */
sfrbit CC12IR;		/* CC12IC.7 */

sfr CC13IC;
sfrbit CC13IE;		/* CC13IC.6 */
sfrbit CC13IR;		/* CC13IC.7 */

sfr CC14IC;
sfrbit CC14IE;		/* CC14IC.6 */
sfrbit CC14IR;		/* CC14IC.7 */

sfr CC15IC;
sfrbit CC15IE;		/* CC15IC.6 */
sfrbit CC15IR;		/* CC15IC.7 */

esfr CC16IC;
esfrbit CC16IE;		/* CC16IC.6 */
esfrbit CC16IR;		/* CC16IC.7 */

esfr CC17IC;
esfrbit CC17IE;		/* CC17IC.6 */
esfrbit CC17IR;		/* CC17IC.7 */

esfr CC18IC;
esfrbit CC18IE;		/* CC18IC.6 */
esfrbit CC18IR;		/* CC18IC.7 */

esfr CC19IC;
esfrbit CC19IE;		/* CC19IC.6 */
esfrbit CC19IR;		/* CC19IC.7 */

esfr CC20IC;
esfrbit CC20IE;		/* CC20IC.6 */
esfrbit CC20IR;		/* CC20IC.7 */

esfr CC21IC;
esfrbit CC21IE;		/* CC21IC.6 */
esfrbit CC21IR;		/* CC21IC.7 */

esfr CC22IC;
esfrbit CC22IE;		/* CC22IC.6 */
esfrbit CC22IR;		/* CC22IC.7 */

esfr CC23IC;
esfrbit CC23IE;		/* CC23IC.6 */
esfrbit CC23IR;		/* CC23IC.7 */

esfr CC24IC;
esfrbit CC24IE;		/* CC24IC.6 */
esfrbit CC24IR;		/* CC24IC.7 */

esfr CC25IC;
esfrbit CC25IE;		/* CC25IC.6 */
esfrbit CC25IR;		/* CC25IC.7 */

esfr CC26IC;
esfrbit CC26IE;		/* CC26IC.6 */
esfrbit CC26IR;		/* CC26IC.7 */

esfr CC27IC;
esfrbit CC27IE;		/* CC27IC.6 */
esfrbit CC27IR;		/* CC27IC.7 */

esfr CC28IC;
esfrbit CC28IE;		/* CC28IC.6 */
esfrbit CC28IR;		/* CC28IC.7 */

esfr CC29IC;
esfrbit CC29IE;		/* CC29IC.6 */
esfrbit CC29IR;		/* CC29IC.7 */

esfr CC30IC;
esfrbit CC30IE;		/* CC30IC.6 */
esfrbit CC30IR;		/* CC30IC.7 */

esfr CC31IC;
esfrbit CC31IE;		/* CC31IC.6 */
esfrbit CC31IR;		/* CC31IC.7 */

sfr CAPREL;
sfr CC0;
sfr CC1;
sfr CC2;
sfr CC3;
sfr CC4;
sfr CC5;
sfr CC6;
sfr CC7;
sfr CC8;
sfr CC9;
sfr CC10;
sfr CC11;
sfr CC12;
sfr CC13;
sfr CC14;
sfr CC15;
sfr CC16;
sfr CC17;
sfr CC18;
sfr CC19;
sfr CC20;
sfr CC21;
sfr CC22;
sfr CC23;
sfr CC24;
sfr CC25;
sfr CC26;
sfr CC27;
sfr CC28;
sfr CC29;
sfr CC30;
sfr CC31;

sfr T0;
sfr T0REL;
sfr T1;
sfr T1REL;
sfr T2;
sfr T3;
sfr T4;
sfr T5;
sfr T6;
esfr T7;
esfr T7REL;
esfr T8;
esfr T8REL;
#endif /* REG167CR_NOCAPCOM */

#ifndef REG167CR_NOPWM
sfr PWMCON0;
sfrbit PTR0;		/* PWMCON0.0 */
sfrbit PTR1;		/* PWMCON0.1 */
sfrbit PTR2;		/* PWMCON0.2 */
sfrbit PTR3;		/* PWMCON0.3 */
sfrbit PTI0;		/* PWMCON0.4 */
sfrbit PTI1;		/* PWMCON0.5 */
sfrbit PTI2;		/* PWMCON0.6 */
sfrbit PTI3;		/* PWMCON0.7 */
sfrbit PIE0;		/* PWMCON0.8 */
sfrbit PIE1;		/* PWMCON0.9 */
sfrbit PIE2;		/* PWMCON0.10 */
sfrbit PIE3;		/* PWMCON0.11 */
sfrbit PIR0;		/* PWMCON0.12 */
sfrbit PIR1;		/* PWMCON0.13 */
sfrbit PIR2;		/* PWMCON0.14 */
sfrbit PIR3;		/* PWMCON0.15 */

sfr PWMCON1;
sfrbit PEN0;		/* PWMCON1.0 */
sfrbit PEN1;		/* PWMCON1.1 */
sfrbit PEN2;		/* PWMCON1.2 */
sfrbit PEN3;		/* PWMCON1.3 */
sfrbit PM0;		/* PWMCON1.4 */
sfrbit PM1;		/* PWMCON1.5 */
sfrbit PM2;		/* PWMCON1.6 */
sfrbit PM3;		/* PWMCON1.7 */
sfrbit PB01;		/* PWMCON1.12 */
sfrbit PS2;		/* PWMCON1.14 */
sfrbit PS3;		/* PWMCON1.15 */

esfr PWMIC;
esfrbit PWMIE;		/* PWMIC.6 */
esfrbit PWMIR;		/* PWMIC.7 */

esfr PP0;
esfr PP1;
esfr PP2;
esfr PP3;
esfr PT0;
esfr PT1;
esfr PT2;
esfr PT3;
sfr PW0;
sfr PW1;
sfr PW2;
sfr PW3;
#endif /* REG167CR_NOPWM */

#ifndef REG167CR_NOADC
sfr ADCON;
sfrbit ADST;		/* ADCON.7 */
sfrbit ADBSY;		/* ADCON.8 */
sfrbit ADWR;		/* ADCON.9 */
sfrbit ADCIN;		/* ADCON.10 */
sfrbit ADCRQ;		/* ADCON.11 */

sfr ADDAT;
esfr ADDAT2;

sfr ADCIC;
sfrbit ADCIE;		/* ADCIC.6 */
sfrbit ADCIR;		/* ADCIC.7 */

sfr ADEIC;
sfrbit ADEIE;		/* ADEIC.6 */
sfrbit ADEIR;		/* ADEIC.7 */
#endif /* REG167CR_NOADC */

#ifndef REG167CR_NOXPER
esfr XP0IC;
esfrbit XP0IE;		/* XP0IC.6 */
esfrbit XP0IR;		/* XP0IC.7 */

esfr XP1IC;
esfrbit XP1IE;		/* XP1IC.6 */
esfrbit XP1IR;		/* XP1IC.7 */

esfr XP2IC;
esfrbit XP2IE;		/* XP2IC.6 */
esfrbit XP2IR;		/* XP2IC.7 */

esfr XP3IC;
esfrbit XP3IE;		/* XP3IC.6 */
esfrbit XP3IR;		/* XP3IC.7 */
#endif /* REG167CR_NOXPER */

#endif /* REG167CR_INC */
