@echo off
cls
echo This installation program will install and upgrade all the neccessary
echo files in the TASKING C166/ST10 toolchain version 6.0 revision 2 to
echo support the latest SIEMENS C161CS/JC/JI and C167CR/CS derivatives.
echo.
echo Backup copies will be made of all files which are upgraded.
echo.
pause

rem
rem Check TASKING C166/ST10 toolchain v6.0r2 location and availability.
rem
if "%1"=="" goto failpdns
if NOT EXIST %1\bin\c166.exe goto failtnf
if NOT EXIST %1\etc\reg167.def goto failtnf
if NOT EXIST %1\include\cstartx.asm goto failtnf

cls
echo Installing new derivative support files...
echo.
echo Installing SIEMENS C161CS register definition file...
copy reg161cs.def %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C161JC register definition file...
copy reg161jc.def %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C161JI register definition file...
copy reg161ji.def %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C167CR register definition file...
copy reg167cr.def %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C167CS register definition file...
copy reg167cs.def %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing register definition files... successfull
echo.
pause

cls
echo Installing new derivative support files...
echo.
echo Installing SIEMENS C161CS header file...
copy reg161cs.h %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C161JC header file...
copy reg161jc.h %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C161JI header file...
copy reg161ji.h %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C167CR header file...
copy reg167cr.h %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Installing SIEMENS C167CS header file...
copy reg167cs.h %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Installing header files... successfull
echo.
pause

cls
echo Generating CrossView Pro register support files...
echo.
echo The following message will be displayed 5 times:
echo WARNING: reg?????.def, line ??: redefinition of register 'TFR' ignored
echo This message is not relevant.
echo.
pause
rem
rem Disable printing the 'rm166' banner on the display.
rem
set COPYRIGHT=TASKING
echo Generating CrossView Pro SIEMENS C161CS register support file...
%1\bin\rm166 -x reg161cs.def -o %1\etc\reg161cs.dat
if ERRORLEVEL 1 goto fail
echo.
echo Generating CrossView Pro SIEMENS C161JC register support file...
%1\bin\rm166 -x reg161jc.def -o %1\etc\reg161jc.dat
if ERRORLEVEL 1 goto fail
echo.
echo Generating CrossView Pro SIEMENS C161JI register support file...
%1\bin\rm166 -x reg161ji.def -o %1\etc\reg161ji.dat
if ERRORLEVEL 1 goto fail
echo.
echo Generating CrossView Pro SIEMENS C167CR register support file...
%1\bin\rm166 -x reg167cr.def -o %1\etc\reg167cr.dat
if ERRORLEVEL 1 goto fail
echo.
echo Generating CrossView Pro SIEMENS C167CS register support file...
%1\bin\rm166 -x reg167cs.def -o %1\etc\reg167cs.dat
if ERRORLEVEL 1 goto fail
echo.
echo Generating CrossView Pro register support files... successfull
echo.
pause

cls
echo Upgrading C Startup code files...
echo.
echo Creating backup copy of C Startup code file cstart.asm...
copy %1\include\cstart.asm %1\include\cstart.bak
if ERRORLEVEL 1 goto fail
echo.
echo Upgrading C Startup code file cstart.asm...
copy cstart.asm %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Creating backup copy of C Startup code file cstartx.asm...
copy %1\include\cstartx.asm %1\include\cstartx.bak
if ERRORLEVEL 1 goto fail
echo.
echo Upgrading C Startup code file cstartx.asm...
copy cstartx.asm %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Creating backup copy of C Startup code file _c_init.asm...
copy %1\include\_c_init.asm %1\include\_c_init.bak
if ERRORLEVEL 1 goto fail
echo.
echo Upgrading C Startup code file _c_init.asm...
copy _c_init.asm %1\include
if ERRORLEVEL 1 goto fail
echo.
echo Upgrading C Startup code files... successfull
echo.
pause

cls
echo Installing CrossView Pro simulator support files...
echo.
echo Installing CrossView Pro SIEMENS C161CS simulator support file...
copy sim161cs.cfg %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing CrossView Pro SIEMENS C161JC simulator support file...
copy sim161jc.cfg %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing CrossView Pro SIEMENS C161JI simulator support file...
copy sim161ji.cfg %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing CrossView Pro SIEMENS C167CR simulator support file...
copy sim167cr.cfg %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing CrossView Pro SIEMENS C167CS simulator support file...
copy sim167cs.cfg %1\etc
if ERRORLEVEL 1 goto fail
echo.
echo Installing CrossView Pro simulator support files... successfull
echo.
pause

cls
echo Upgrading EDE support files...
echo.
echo Creating backup copy of EDE support file c166.dol...
copy %1\etc\c166.dol %1\etc\c166.bak
if ERRORLEVEL 1 goto fail
if NOT EXIST %1\bin\cp166.exe goto doilc
echo.
echo Upgrading EDE C++ Compiler package support file c166.dol...
copy c166cpp.dol %1\etc\c166.dol
if ERRORLEVEL 1 goto fail
goto contdoil
:doilc
echo.
echo Upgrading EDE C Compiler package support file c166.dol...
copy c166c.dol %1\etc\c166.dol
if ERRORLEVEL 1 goto fail
:contdoil
echo.
echo Upgrading EDE support files... successfull
echo.
pause
goto success

rem
rem Failure/Success handling
rem
:failpdns
cls
echo.
echo Please specify the directory where the TASKING C166/ST10 toolchain 
echo verison 6.0 revision 2 is installed.
echo.
echo Example: upgrade c:\c166
echo.
pause
goto end

:failtnf
cls
echo.
echo The TASKING C166/ST10 toolchain version 6.0 revision 2 was not
echo found in the directory %1.
echo.
goto end

:fail
echo.
echo An error has occurred. If the disk is full or you do not have 
echo write permissions, then contact your local system administrator.
echo.
goto end

:success
cls
echo Upgrade successfull.
echo.

:end